from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from src.api.routes import (auth, flights, health, notifications,
                            subscriptions, users)

import logging
import graypy
from src.api.config import LOGS_UPD

my_logger = logging.getLogger('test_logger')
my_logger.setLevel(logging.DEBUG)

handler = graypy.GELFUDPHandler("fids_logstash", 12201)
my_logger.addHandler(handler)

# my_logger.debug('Hello Graylog.')

app = FastAPI(title="Flights Information API")
app.include_router(flights.router, prefix="/flights")
app.include_router(health.router, prefix="/health")
app.include_router(auth.router, prefix="/auth")
app.include_router(users.router, prefix="/users")
app.include_router(subscriptions.router, prefix="/subscriptions")
app.include_router(notifications.router, prefix="/notifications")
app.add_middleware(
    CORSMiddleware,
    allow_origins=[
        "https://fids.slc.ar",
        "https://airport.fids.slc.ar",
        "http://localhost:8080",
        "http://localhost:8081",
        "http://localhost:3000",
    ],
    allow_credentials=True,
    allow_methods=["POST", "GET", "PUT", "DELETE", "OPTIONS"],
    allow_headers=["*"],
)
